"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .datacollection import DataCollection
from .openairesponsesincludable import OpenAIResponsesIncludable
from .openairesponsesprompt import OpenAIResponsesPrompt, OpenAIResponsesPromptTypedDict
from .openairesponsestoolchoice_union import (
    OpenAIResponsesToolChoiceUnion,
    OpenAIResponsesToolChoiceUnionTypedDict,
)
from .openresponsesinput import OpenResponsesInput, OpenResponsesInputTypedDict
from .openresponsesreasoningconfig import (
    OpenResponsesReasoningConfig,
    OpenResponsesReasoningConfigTypedDict,
)
from .openresponsesresponsetext import (
    OpenResponsesResponseText,
    OpenResponsesResponseTextTypedDict,
)
from .openresponseswebsearch20250826tool import (
    OpenResponsesWebSearch20250826Tool,
    OpenResponsesWebSearch20250826ToolTypedDict,
)
from .openresponseswebsearchpreview20250311tool import (
    OpenResponsesWebSearchPreview20250311Tool,
    OpenResponsesWebSearchPreview20250311ToolTypedDict,
)
from .openresponseswebsearchpreviewtool import (
    OpenResponsesWebSearchPreviewTool,
    OpenResponsesWebSearchPreviewToolTypedDict,
)
from .openresponseswebsearchtool import (
    OpenResponsesWebSearchTool,
    OpenResponsesWebSearchToolTypedDict,
)
from .pdfparseroptions import PDFParserOptions, PDFParserOptionsTypedDict
from .preferredmaxlatency import PreferredMaxLatency, PreferredMaxLatencyTypedDict
from .preferredminthroughput import (
    PreferredMinThroughput,
    PreferredMinThroughputTypedDict,
)
from .providername import ProviderName
from .providersort import ProviderSort
from .providersortconfig import ProviderSortConfig, ProviderSortConfigTypedDict
from .quantization import Quantization
from .responsesoutputmodality import ResponsesOutputModality
from .websearchengine import WebSearchEngine
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_const, validate_open_enum
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OpenResponsesRequestType = Literal["function",]


class OpenResponsesRequestToolFunctionTypedDict(TypedDict):
    r"""Function tool definition"""

    type: OpenResponsesRequestType
    name: str
    parameters: Nullable[Dict[str, Nullable[Any]]]
    description: NotRequired[Nullable[str]]
    strict: NotRequired[Nullable[bool]]


class OpenResponsesRequestToolFunction(BaseModel):
    r"""Function tool definition"""

    type: OpenResponsesRequestType

    name: str

    parameters: Nullable[Dict[str, Nullable[Any]]]

    description: OptionalNullable[str] = UNSET

    strict: OptionalNullable[bool] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "strict"]
        nullable_fields = ["description", "strict", "parameters"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


OpenResponsesRequestToolUnionTypedDict = TypeAliasType(
    "OpenResponsesRequestToolUnionTypedDict",
    Union[
        OpenResponsesWebSearchPreviewToolTypedDict,
        OpenResponsesWebSearchPreview20250311ToolTypedDict,
        OpenResponsesWebSearchToolTypedDict,
        OpenResponsesWebSearch20250826ToolTypedDict,
        OpenResponsesRequestToolFunctionTypedDict,
    ],
)


OpenResponsesRequestToolUnion = Annotated[
    Union[
        Annotated[OpenResponsesRequestToolFunction, Tag("function")],
        Annotated[OpenResponsesWebSearchPreviewTool, Tag("web_search_preview")],
        Annotated[
            OpenResponsesWebSearchPreview20250311Tool,
            Tag("web_search_preview_2025_03_11"),
        ],
        Annotated[OpenResponsesWebSearchTool, Tag("web_search")],
        Annotated[OpenResponsesWebSearch20250826Tool, Tag("web_search_2025_08_26")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


OpenResponsesRequestImageConfigTypedDict = TypeAliasType(
    "OpenResponsesRequestImageConfigTypedDict", Union[str, float]
)


OpenResponsesRequestImageConfig = TypeAliasType(
    "OpenResponsesRequestImageConfig", Union[str, float]
)


ServiceTier = Literal["auto",]


Truncation = Union[
    Literal[
        "auto",
        "disabled",
    ],
    UnrecognizedStr,
]


OpenResponsesRequestOrderTypedDict = TypeAliasType(
    "OpenResponsesRequestOrderTypedDict", Union[ProviderName, str]
)


OpenResponsesRequestOrder = TypeAliasType(
    "OpenResponsesRequestOrder",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


OpenResponsesRequestOnlyTypedDict = TypeAliasType(
    "OpenResponsesRequestOnlyTypedDict", Union[ProviderName, str]
)


OpenResponsesRequestOnly = TypeAliasType(
    "OpenResponsesRequestOnly",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


OpenResponsesRequestIgnoreTypedDict = TypeAliasType(
    "OpenResponsesRequestIgnoreTypedDict", Union[ProviderName, str]
)


OpenResponsesRequestIgnore = TypeAliasType(
    "OpenResponsesRequestIgnore",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


OpenResponsesRequestSortTypedDict = TypeAliasType(
    "OpenResponsesRequestSortTypedDict",
    Union[ProviderSortConfigTypedDict, ProviderSort, Any],
)
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


OpenResponsesRequestSort = TypeAliasType(
    "OpenResponsesRequestSort",
    Union[
        ProviderSortConfig,
        Annotated[ProviderSort, PlainValidator(validate_open_enum(False))],
        Any,
    ],
)
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


class OpenResponsesRequestMaxPriceTypedDict(TypedDict):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: NotRequired[str]
    r"""A value in string format that is a large number"""
    completion: NotRequired[str]
    r"""A value in string format that is a large number"""
    image: NotRequired[str]
    r"""A value in string format that is a large number"""
    audio: NotRequired[str]
    r"""A value in string format that is a large number"""
    request: NotRequired[str]
    r"""A value in string format that is a large number"""


class OpenResponsesRequestMaxPrice(BaseModel):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: Optional[str] = None
    r"""A value in string format that is a large number"""

    completion: Optional[str] = None
    r"""A value in string format that is a large number"""

    image: Optional[str] = None
    r"""A value in string format that is a large number"""

    audio: Optional[str] = None
    r"""A value in string format that is a large number"""

    request: Optional[str] = None
    r"""A value in string format that is a large number"""


class OpenResponsesRequestProviderTypedDict(TypedDict):
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    allow_fallbacks: NotRequired[Nullable[bool]]
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """
    require_parameters: NotRequired[Nullable[bool]]
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""
    data_collection: NotRequired[Nullable[DataCollection]]
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """
    zdr: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""
    enforce_distillable_text: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""
    order: NotRequired[Nullable[List[OpenResponsesRequestOrderTypedDict]]]
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""
    only: NotRequired[Nullable[List[OpenResponsesRequestOnlyTypedDict]]]
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""
    ignore: NotRequired[Nullable[List[OpenResponsesRequestIgnoreTypedDict]]]
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""
    quantizations: NotRequired[Nullable[List[Quantization]]]
    r"""A list of quantization levels to filter the provider by."""
    sort: NotRequired[Nullable[OpenResponsesRequestSortTypedDict]]
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""
    max_price: NotRequired[OpenResponsesRequestMaxPriceTypedDict]
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""
    preferred_min_throughput: NotRequired[Nullable[PreferredMinThroughputTypedDict]]
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""
    preferred_max_latency: NotRequired[Nullable[PreferredMaxLatencyTypedDict]]
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""


class OpenResponsesRequestProvider(BaseModel):
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    allow_fallbacks: OptionalNullable[bool] = UNSET
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """

    require_parameters: OptionalNullable[bool] = UNSET
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""

    data_collection: Annotated[
        OptionalNullable[DataCollection], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """

    zdr: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""

    enforce_distillable_text: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""

    order: OptionalNullable[List[OpenResponsesRequestOrder]] = UNSET
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""

    only: OptionalNullable[List[OpenResponsesRequestOnly]] = UNSET
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""

    ignore: OptionalNullable[List[OpenResponsesRequestIgnore]] = UNSET
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""

    quantizations: OptionalNullable[
        List[Annotated[Quantization, PlainValidator(validate_open_enum(False))]]
    ] = UNSET
    r"""A list of quantization levels to filter the provider by."""

    sort: OptionalNullable[OpenResponsesRequestSort] = UNSET
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""

    max_price: Optional[OpenResponsesRequestMaxPrice] = None
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    preferred_min_throughput: OptionalNullable[PreferredMinThroughput] = UNSET
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    preferred_max_latency: OptionalNullable[PreferredMaxLatency] = UNSET
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "max_price",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        nullable_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


IDResponseHealing = Literal["response-healing",]


class OpenResponsesRequestPluginResponseHealingTypedDict(TypedDict):
    id: IDResponseHealing
    enabled: NotRequired[bool]
    r"""Set to false to disable the response-healing plugin for this request. Defaults to true."""


class OpenResponsesRequestPluginResponseHealing(BaseModel):
    id: IDResponseHealing

    enabled: Optional[bool] = None
    r"""Set to false to disable the response-healing plugin for this request. Defaults to true."""


IDFileParser = Literal["file-parser",]


class OpenResponsesRequestPluginFileParserTypedDict(TypedDict):
    id: IDFileParser
    enabled: NotRequired[bool]
    r"""Set to false to disable the file-parser plugin for this request. Defaults to true."""
    pdf: NotRequired[PDFParserOptionsTypedDict]
    r"""Options for PDF parsing."""


class OpenResponsesRequestPluginFileParser(BaseModel):
    id: IDFileParser

    enabled: Optional[bool] = None
    r"""Set to false to disable the file-parser plugin for this request. Defaults to true."""

    pdf: Optional[PDFParserOptions] = None
    r"""Options for PDF parsing."""


IDWeb = Literal["web",]


class OpenResponsesRequestPluginWebTypedDict(TypedDict):
    id: IDWeb
    enabled: NotRequired[bool]
    r"""Set to false to disable the web-search plugin for this request. Defaults to true."""
    max_results: NotRequired[float]
    search_prompt: NotRequired[str]
    engine: NotRequired[WebSearchEngine]
    r"""The search engine to use for web search."""


class OpenResponsesRequestPluginWeb(BaseModel):
    id: IDWeb

    enabled: Optional[bool] = None
    r"""Set to false to disable the web-search plugin for this request. Defaults to true."""

    max_results: Optional[float] = None

    search_prompt: Optional[str] = None

    engine: Annotated[
        Optional[WebSearchEngine], PlainValidator(validate_open_enum(False))
    ] = None
    r"""The search engine to use for web search."""


IDModeration = Literal["moderation",]


class OpenResponsesRequestPluginModerationTypedDict(TypedDict):
    id: IDModeration


class OpenResponsesRequestPluginModeration(BaseModel):
    id: IDModeration


IDAutoRouter = Literal["auto-router",]


class OpenResponsesRequestPluginAutoRouterTypedDict(TypedDict):
    id: IDAutoRouter
    enabled: NotRequired[bool]
    r"""Set to false to disable the auto-router plugin for this request. Defaults to true."""
    allowed_models: NotRequired[List[str]]
    r"""List of model patterns to filter which models the auto-router can route between. Supports wildcards (e.g., \"anthropic/*\" matches all Anthropic models). When not specified, uses the default supported models list."""


class OpenResponsesRequestPluginAutoRouter(BaseModel):
    id: IDAutoRouter

    enabled: Optional[bool] = None
    r"""Set to false to disable the auto-router plugin for this request. Defaults to true."""

    allowed_models: Optional[List[str]] = None
    r"""List of model patterns to filter which models the auto-router can route between. Supports wildcards (e.g., \"anthropic/*\" matches all Anthropic models). When not specified, uses the default supported models list."""


OpenResponsesRequestPluginUnionTypedDict = TypeAliasType(
    "OpenResponsesRequestPluginUnionTypedDict",
    Union[
        OpenResponsesRequestPluginModerationTypedDict,
        OpenResponsesRequestPluginResponseHealingTypedDict,
        OpenResponsesRequestPluginAutoRouterTypedDict,
        OpenResponsesRequestPluginFileParserTypedDict,
        OpenResponsesRequestPluginWebTypedDict,
    ],
)


OpenResponsesRequestPluginUnion = Annotated[
    Union[
        Annotated[OpenResponsesRequestPluginAutoRouter, Tag("auto-router")],
        Annotated[OpenResponsesRequestPluginModeration, Tag("moderation")],
        Annotated[OpenResponsesRequestPluginWeb, Tag("web")],
        Annotated[OpenResponsesRequestPluginFileParser, Tag("file-parser")],
        Annotated[OpenResponsesRequestPluginResponseHealing, Tag("response-healing")],
    ],
    Discriminator(lambda m: get_discriminator(m, "id", "id")),
]


class OpenResponsesRequestTypedDict(TypedDict):
    r"""Request schema for Responses endpoint"""

    input: NotRequired[OpenResponsesInputTypedDict]
    r"""Input for a response request - can be a string or array of items"""
    instructions: NotRequired[Nullable[str]]
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""
    tools: NotRequired[List[OpenResponsesRequestToolUnionTypedDict]]
    tool_choice: NotRequired[OpenAIResponsesToolChoiceUnionTypedDict]
    parallel_tool_calls: NotRequired[Nullable[bool]]
    model: NotRequired[str]
    models: NotRequired[List[str]]
    text: NotRequired[OpenResponsesResponseTextTypedDict]
    r"""Text output configuration including format and verbosity"""
    reasoning: NotRequired[Nullable[OpenResponsesReasoningConfigTypedDict]]
    r"""Configuration for reasoning mode in the response"""
    max_output_tokens: NotRequired[Nullable[float]]
    temperature: NotRequired[Nullable[float]]
    top_p: NotRequired[Nullable[float]]
    top_logprobs: NotRequired[Nullable[int]]
    max_tool_calls: NotRequired[Nullable[int]]
    presence_penalty: NotRequired[Nullable[float]]
    frequency_penalty: NotRequired[Nullable[float]]
    top_k: NotRequired[float]
    image_config: NotRequired[Dict[str, OpenResponsesRequestImageConfigTypedDict]]
    r"""Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/features/multimodal/image-generation for more details."""
    modalities: NotRequired[List[ResponsesOutputModality]]
    r"""Output modalities for the response. Supported values are \"text\" and \"image\"."""
    prompt_cache_key: NotRequired[Nullable[str]]
    previous_response_id: NotRequired[Nullable[str]]
    prompt: NotRequired[Nullable[OpenAIResponsesPromptTypedDict]]
    include: NotRequired[Nullable[List[OpenAIResponsesIncludable]]]
    background: NotRequired[Nullable[bool]]
    safety_identifier: NotRequired[Nullable[str]]
    store: Literal[False]
    service_tier: NotRequired[ServiceTier]
    truncation: NotRequired[Nullable[Truncation]]
    stream: NotRequired[bool]
    provider: NotRequired[Nullable[OpenResponsesRequestProviderTypedDict]]
    r"""When multiple model providers are available, optionally indicate your routing preference."""
    plugins: NotRequired[List[OpenResponsesRequestPluginUnionTypedDict]]
    r"""Plugins you want to enable for this request, including their settings."""
    user: NotRequired[str]
    r"""A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters."""
    session_id: NotRequired[str]
    r"""A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters."""


class OpenResponsesRequest(BaseModel):
    r"""Request schema for Responses endpoint"""

    input: Optional[OpenResponsesInput] = None
    r"""Input for a response request - can be a string or array of items"""

    instructions: OptionalNullable[str] = UNSET

    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""

    tools: Optional[List[OpenResponsesRequestToolUnion]] = None

    tool_choice: Optional[OpenAIResponsesToolChoiceUnion] = None

    parallel_tool_calls: OptionalNullable[bool] = UNSET

    model: Optional[str] = None

    models: Optional[List[str]] = None

    text: Optional[OpenResponsesResponseText] = None
    r"""Text output configuration including format and verbosity"""

    reasoning: OptionalNullable[OpenResponsesReasoningConfig] = UNSET
    r"""Configuration for reasoning mode in the response"""

    max_output_tokens: OptionalNullable[float] = UNSET

    temperature: OptionalNullable[float] = UNSET

    top_p: OptionalNullable[float] = UNSET

    top_logprobs: OptionalNullable[int] = UNSET

    max_tool_calls: OptionalNullable[int] = UNSET

    presence_penalty: OptionalNullable[float] = UNSET

    frequency_penalty: OptionalNullable[float] = UNSET

    top_k: Optional[float] = None

    image_config: Optional[Dict[str, OpenResponsesRequestImageConfig]] = None
    r"""Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/features/multimodal/image-generation for more details."""

    modalities: Optional[
        List[
            Annotated[
                ResponsesOutputModality, PlainValidator(validate_open_enum(False))
            ]
        ]
    ] = None
    r"""Output modalities for the response. Supported values are \"text\" and \"image\"."""

    prompt_cache_key: OptionalNullable[str] = UNSET

    previous_response_id: OptionalNullable[str] = UNSET

    prompt: OptionalNullable[OpenAIResponsesPrompt] = UNSET

    include: OptionalNullable[
        List[
            Annotated[
                OpenAIResponsesIncludable, PlainValidator(validate_open_enum(False))
            ]
        ]
    ] = UNSET

    background: OptionalNullable[bool] = UNSET

    safety_identifier: OptionalNullable[str] = UNSET

    STORE: Annotated[
        Annotated[Optional[Literal[False]], AfterValidator(validate_const(False))],
        pydantic.Field(alias="store"),
    ] = False

    service_tier: Optional[ServiceTier] = "auto"

    truncation: Annotated[
        OptionalNullable[Truncation], PlainValidator(validate_open_enum(False))
    ] = UNSET

    stream: Optional[bool] = False

    provider: OptionalNullable[OpenResponsesRequestProvider] = UNSET
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    plugins: Optional[List[OpenResponsesRequestPluginUnion]] = None
    r"""Plugins you want to enable for this request, including their settings."""

    user: Optional[str] = None
    r"""A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters."""

    session_id: Optional[str] = None
    r"""A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "input",
            "instructions",
            "metadata",
            "tools",
            "tool_choice",
            "parallel_tool_calls",
            "model",
            "models",
            "text",
            "reasoning",
            "max_output_tokens",
            "temperature",
            "top_p",
            "top_logprobs",
            "max_tool_calls",
            "presence_penalty",
            "frequency_penalty",
            "top_k",
            "image_config",
            "modalities",
            "prompt_cache_key",
            "previous_response_id",
            "prompt",
            "include",
            "background",
            "safety_identifier",
            "store",
            "service_tier",
            "truncation",
            "stream",
            "provider",
            "plugins",
            "user",
            "session_id",
        ]
        nullable_fields = [
            "instructions",
            "metadata",
            "parallel_tool_calls",
            "reasoning",
            "max_output_tokens",
            "temperature",
            "top_p",
            "top_logprobs",
            "max_tool_calls",
            "presence_penalty",
            "frequency_penalty",
            "prompt_cache_key",
            "previous_response_id",
            "prompt",
            "include",
            "background",
            "safety_identifier",
            "truncation",
            "provider",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .imagegenerationstatus import ImageGenerationStatus
from openrouter.types import BaseModel, Nullable, OptionalNullable, UNSET_SENTINEL
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal
from typing_extensions import Annotated, NotRequired, TypedDict


OutputItemImageGenerationCallType = Literal["image_generation_call",]


class OutputItemImageGenerationCallTypedDict(TypedDict):
    type: OutputItemImageGenerationCallType
    id: str
    status: ImageGenerationStatus
    result: NotRequired[Nullable[str]]


class OutputItemImageGenerationCall(BaseModel):
    type: OutputItemImageGenerationCallType

    id: str

    status: Annotated[ImageGenerationStatus, PlainValidator(validate_open_enum(False))]

    result: OptionalNullable[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["result"]
        nullable_fields = ["result"]
        null_default_fields = ["result"]

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from openrouter.utils import FieldMetadata, HeaderMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GlobalsTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """


class Globals(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ChatStreamOptionsTypedDict(TypedDict):
    include_usage: NotRequired[bool]


class ChatStreamOptions(BaseModel):
    include_usage: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesannotation import (
    OpenAIResponsesAnnotation,
    OpenAIResponsesAnnotationTypedDict,
)
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .openresponseserrorevent import (
    OpenResponsesErrorEvent,
    OpenResponsesErrorEventTypedDict,
)
from .openresponsesimagegencallcompleted import (
    OpenResponsesImageGenCallCompleted,
    OpenResponsesImageGenCallCompletedTypedDict,
)
from .openresponsesimagegencallgenerating import (
    OpenResponsesImageGenCallGenerating,
    OpenResponsesImageGenCallGeneratingTypedDict,
)
from .openresponsesimagegencallinprogress import (
    OpenResponsesImageGenCallInProgress,
    OpenResponsesImageGenCallInProgressTypedDict,
)
from .openresponsesimagegencallpartialimage import (
    OpenResponsesImageGenCallPartialImage,
    OpenResponsesImageGenCallPartialImageTypedDict,
)
from .openresponseslogprobs import OpenResponsesLogProbs, OpenResponsesLogProbsTypedDict
from .openresponsesnonstreamingresponse import (
    OpenResponsesNonStreamingResponse,
    OpenResponsesNonStreamingResponseTypedDict,
)
from .openresponsesreasoningdeltaevent import (
    OpenResponsesReasoningDeltaEvent,
    OpenResponsesReasoningDeltaEventTypedDict,
)
from .openresponsesreasoningdoneevent import (
    OpenResponsesReasoningDoneEvent,
    OpenResponsesReasoningDoneEventTypedDict,
)
from .openresponsesreasoningsummarypartaddedevent import (
    OpenResponsesReasoningSummaryPartAddedEvent,
    OpenResponsesReasoningSummaryPartAddedEventTypedDict,
)
from .openresponsesreasoningsummarytextdeltaevent import (
    OpenResponsesReasoningSummaryTextDeltaEvent,
    OpenResponsesReasoningSummaryTextDeltaEventTypedDict,
)
from .openresponsesreasoningsummarytextdoneevent import (
    OpenResponsesReasoningSummaryTextDoneEvent,
    OpenResponsesReasoningSummaryTextDoneEventTypedDict,
)
from .reasoningsummarytext import ReasoningSummaryText, ReasoningSummaryTextTypedDict
from .reasoningtextcontent import ReasoningTextContent, ReasoningTextContentTypedDict
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from .responsesoutputitem import ResponsesOutputItem, ResponsesOutputItemTypedDict
from openrouter.types import BaseModel
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import List, Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


TypeResponseReasoningSummaryPartDone = Literal["response.reasoning_summary_part.done",]


class OpenResponsesStreamEventResponseReasoningSummaryPartDoneTypedDict(TypedDict):
    r"""Event emitted when a reasoning summary part is complete"""

    type: TypeResponseReasoningSummaryPartDone
    output_index: float
    item_id: str
    summary_index: float
    part: ReasoningSummaryTextTypedDict
    sequence_number: float


class OpenResponsesStreamEventResponseReasoningSummaryPartDone(BaseModel):
    r"""Event emitted when a reasoning summary part is complete"""

    type: TypeResponseReasoningSummaryPartDone

    output_index: float

    item_id: str

    summary_index: float

    part: ReasoningSummaryText

    sequence_number: float


TypeResponseFunctionCallArgumentsDone = Literal[
    "response.function_call_arguments.done",
]


class OpenResponsesStreamEventResponseFunctionCallArgumentsDoneTypedDict(TypedDict):
    r"""Event emitted when function call arguments streaming is complete"""

    type: TypeResponseFunctionCallArgumentsDone
    item_id: str
    output_index: float
    name: str
    arguments: str
    sequence_number: float


class OpenResponsesStreamEventResponseFunctionCallArgumentsDone(BaseModel):
    r"""Event emitted when function call arguments streaming is complete"""

    type: TypeResponseFunctionCallArgumentsDone

    item_id: str

    output_index: float

    name: str

    arguments: str

    sequence_number: float


TypeResponseFunctionCallArgumentsDelta = Literal[
    "response.function_call_arguments.delta",
]


class OpenResponsesStreamEventResponseFunctionCallArgumentsDeltaTypedDict(TypedDict):
    r"""Event emitted when function call arguments are being streamed"""

    type: TypeResponseFunctionCallArgumentsDelta
    item_id: str
    output_index: float
    delta: str
    sequence_number: float


class OpenResponsesStreamEventResponseFunctionCallArgumentsDelta(BaseModel):
    r"""Event emitted when function call arguments are being streamed"""

    type: TypeResponseFunctionCallArgumentsDelta

    item_id: str

    output_index: float

    delta: str

    sequence_number: float


TypeResponseOutputTextAnnotationAdded = Literal[
    "response.output_text.annotation.added",
]


class OpenResponsesStreamEventResponseOutputTextAnnotationAddedTypedDict(TypedDict):
    r"""Event emitted when a text annotation is added to output"""

    type: TypeResponseOutputTextAnnotationAdded
    output_index: float
    item_id: str
    content_index: float
    sequence_number: float
    annotation_index: float
    annotation: OpenAIResponsesAnnotationTypedDict


class OpenResponsesStreamEventResponseOutputTextAnnotationAdded(BaseModel):
    r"""Event emitted when a text annotation is added to output"""

    type: TypeResponseOutputTextAnnotationAdded

    output_index: float

    item_id: str

    content_index: float

    sequence_number: float

    annotation_index: float

    annotation: OpenAIResponsesAnnotation


TypeResponseRefusalDone = Literal["response.refusal.done",]


class OpenResponsesStreamEventResponseRefusalDoneTypedDict(TypedDict):
    r"""Event emitted when refusal streaming is complete"""

    type: TypeResponseRefusalDone
    output_index: float
    item_id: str
    content_index: float
    refusal: str
    sequence_number: float


class OpenResponsesStreamEventResponseRefusalDone(BaseModel):
    r"""Event emitted when refusal streaming is complete"""

    type: TypeResponseRefusalDone

    output_index: float

    item_id: str

    content_index: float

    refusal: str

    sequence_number: float


TypeResponseRefusalDelta = Literal["response.refusal.delta",]


class OpenResponsesStreamEventResponseRefusalDeltaTypedDict(TypedDict):
    r"""Event emitted when a refusal delta is streamed"""

    type: TypeResponseRefusalDelta
    output_index: float
    item_id: str
    content_index: float
    delta: str
    sequence_number: float


class OpenResponsesStreamEventResponseRefusalDelta(BaseModel):
    r"""Event emitted when a refusal delta is streamed"""

    type: TypeResponseRefusalDelta

    output_index: float

    item_id: str

    content_index: float

    delta: str

    sequence_number: float


TypeResponseOutputTextDone = Literal["response.output_text.done",]


class OpenResponsesStreamEventResponseOutputTextDoneTypedDict(TypedDict):
    r"""Event emitted when text streaming is complete"""

    type: TypeResponseOutputTextDone
    output_index: float
    item_id: str
    content_index: float
    text: str
    sequence_number: float
    logprobs: List[OpenResponsesLogProbsTypedDict]


class OpenResponsesStreamEventResponseOutputTextDone(BaseModel):
    r"""Event emitted when text streaming is complete"""

    type: TypeResponseOutputTextDone

    output_index: float

    item_id: str

    content_index: float

    text: str

    sequence_number: float

    logprobs: List[OpenResponsesLogProbs]


TypeResponseOutputTextDelta = Literal["response.output_text.delta",]


class OpenResponsesStreamEventResponseOutputTextDeltaTypedDict(TypedDict):
    r"""Event emitted when a text delta is streamed"""

    type: TypeResponseOutputTextDelta
    logprobs: List[OpenResponsesLogProbsTypedDict]
    output_index: float
    item_id: str
    content_index: float
    delta: str
    sequence_number: float


class OpenResponsesStreamEventResponseOutputTextDelta(BaseModel):
    r"""Event emitted when a text delta is streamed"""

    type: TypeResponseOutputTextDelta

    logprobs: List[OpenResponsesLogProbs]

    output_index: float

    item_id: str

    content_index: float

    delta: str

    sequence_number: float


TypeResponseContentPartDone = Literal["response.content_part.done",]


Part2TypedDict = TypeAliasType(
    "Part2TypedDict",
    Union[
        ReasoningTextContentTypedDict,
        OpenAIResponsesRefusalContentTypedDict,
        ResponseOutputTextTypedDict,
    ],
)


Part2 = Annotated[
    Union[
        Annotated[ResponseOutputText, Tag("output_text")],
        Annotated[ReasoningTextContent, Tag("reasoning_text")],
        Annotated[OpenAIResponsesRefusalContent, Tag("refusal")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class OpenResponsesStreamEventResponseContentPartDoneTypedDict(TypedDict):
    r"""Event emitted when a content part is complete"""

    type: TypeResponseContentPartDone
    output_index: float
    item_id: str
    content_index: float
    part: Part2TypedDict
    sequence_number: float


class OpenResponsesStreamEventResponseContentPartDone(BaseModel):
    r"""Event emitted when a content part is complete"""

    type: TypeResponseContentPartDone

    output_index: float

    item_id: str

    content_index: float

    part: Part2

    sequence_number: float


TypeResponseContentPartAdded = Literal["response.content_part.added",]


Part1TypedDict = TypeAliasType(
    "Part1TypedDict",
    Union[
        ReasoningTextContentTypedDict,
        OpenAIResponsesRefusalContentTypedDict,
        ResponseOutputTextTypedDict,
    ],
)


Part1 = Annotated[
    Union[
        Annotated[ResponseOutputText, Tag("output_text")],
        Annotated[ReasoningTextContent, Tag("reasoning_text")],
        Annotated[OpenAIResponsesRefusalContent, Tag("refusal")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class OpenResponsesStreamEventResponseContentPartAddedTypedDict(TypedDict):
    r"""Event emitted when a new content part is added to an output item"""

    type: TypeResponseContentPartAdded
    output_index: float
    item_id: str
    content_index: float
    part: Part1TypedDict
    sequence_number: float


class OpenResponsesStreamEventResponseContentPartAdded(BaseModel):
    r"""Event emitted when a new content part is added to an output item"""

    type: TypeResponseContentPartAdded

    output_index: float

    item_id: str

    content_index: float

    part: Part1

    sequence_number: float


TypeResponseOutputItemDone = Literal["response.output_item.done",]


class OpenResponsesStreamEventResponseOutputItemDoneTypedDict(TypedDict):
    r"""Event emitted when an output item is complete"""

    type: TypeResponseOutputItemDone
    output_index: float
    item: ResponsesOutputItemTypedDict
    r"""An output item from the response"""
    sequence_number: float


class OpenResponsesStreamEventResponseOutputItemDone(BaseModel):
    r"""Event emitted when an output item is complete"""

    type: TypeResponseOutputItemDone

    output_index: float

    item: ResponsesOutputItem
    r"""An output item from the response"""

    sequence_number: float


TypeResponseOutputItemAdded = Literal["response.output_item.added",]


class OpenResponsesStreamEventResponseOutputItemAddedTypedDict(TypedDict):
    r"""Event emitted when a new output item is added to the response"""

    type: TypeResponseOutputItemAdded
    output_index: float
    item: ResponsesOutputItemTypedDict
    r"""An output item from the response"""
    sequence_number: float


class OpenResponsesStreamEventResponseOutputItemAdded(BaseModel):
    r"""Event emitted when a new output item is added to the response"""

    type: TypeResponseOutputItemAdded

    output_index: float

    item: ResponsesOutputItem
    r"""An output item from the response"""

    sequence_number: float


TypeResponseFailed = Literal["response.failed",]


class OpenResponsesStreamEventResponseFailedTypedDict(TypedDict):
    r"""Event emitted when a response has failed"""

    type: TypeResponseFailed
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseFailed(BaseModel):
    r"""Event emitted when a response has failed"""

    type: TypeResponseFailed

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseIncomplete = Literal["response.incomplete",]


class OpenResponsesStreamEventResponseIncompleteTypedDict(TypedDict):
    r"""Event emitted when a response is incomplete"""

    type: TypeResponseIncomplete
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseIncomplete(BaseModel):
    r"""Event emitted when a response is incomplete"""

    type: TypeResponseIncomplete

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseCompleted = Literal["response.completed",]


class OpenResponsesStreamEventResponseCompletedTypedDict(TypedDict):
    r"""Event emitted when a response has completed successfully"""

    type: TypeResponseCompleted
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseCompleted(BaseModel):
    r"""Event emitted when a response has completed successfully"""

    type: TypeResponseCompleted

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseInProgress = Literal["response.in_progress",]


class OpenResponsesStreamEventResponseInProgressTypedDict(TypedDict):
    r"""Event emitted when a response is in progress"""

    type: TypeResponseInProgress
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseInProgress(BaseModel):
    r"""Event emitted when a response is in progress"""

    type: TypeResponseInProgress

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseCreated = Literal["response.created",]


class OpenResponsesStreamEventResponseCreatedTypedDict(TypedDict):
    r"""Event emitted when a response is created"""

    type: TypeResponseCreated
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseCreated(BaseModel):
    r"""Event emitted when a response is created"""

    type: TypeResponseCreated

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


OpenResponsesStreamEventTypedDict = TypeAliasType(
    "OpenResponsesStreamEventTypedDict",
    Union[
        OpenResponsesStreamEventResponseCreatedTypedDict,
        OpenResponsesStreamEventResponseInProgressTypedDict,
        OpenResponsesStreamEventResponseCompletedTypedDict,
        OpenResponsesStreamEventResponseIncompleteTypedDict,
        OpenResponsesStreamEventResponseFailedTypedDict,
        OpenResponsesStreamEventResponseOutputItemAddedTypedDict,
        OpenResponsesStreamEventResponseOutputItemDoneTypedDict,
        OpenResponsesImageGenCallCompletedTypedDict,
        OpenResponsesImageGenCallGeneratingTypedDict,
        OpenResponsesImageGenCallInProgressTypedDict,
        OpenResponsesErrorEventTypedDict,
        OpenResponsesStreamEventResponseFunctionCallArgumentsDeltaTypedDict,
        OpenResponsesStreamEventResponseRefusalDeltaTypedDict,
        OpenResponsesReasoningSummaryPartAddedEventTypedDict,
        OpenResponsesStreamEventResponseContentPartAddedTypedDict,
        OpenResponsesImageGenCallPartialImageTypedDict,
        OpenResponsesStreamEventResponseFunctionCallArgumentsDoneTypedDict,
        OpenResponsesReasoningDeltaEventTypedDict,
        OpenResponsesReasoningDoneEventTypedDict,
        OpenResponsesStreamEventResponseRefusalDoneTypedDict,
        OpenResponsesStreamEventResponseReasoningSummaryPartDoneTypedDict,
        OpenResponsesReasoningSummaryTextDeltaEventTypedDict,
        OpenResponsesReasoningSummaryTextDoneEventTypedDict,
        OpenResponsesStreamEventResponseContentPartDoneTypedDict,
        OpenResponsesStreamEventResponseOutputTextDeltaTypedDict,
        OpenResponsesStreamEventResponseOutputTextDoneTypedDict,
        OpenResponsesStreamEventResponseOutputTextAnnotationAddedTypedDict,
    ],
)
r"""Union of all possible event types emitted during response streaming"""


OpenResponsesStreamEvent = Annotated[
    Union[
        Annotated[OpenResponsesStreamEventResponseCreated, Tag("response.created")],
        Annotated[
            OpenResponsesStreamEventResponseInProgress, Tag("response.in_progress")
        ],
        Annotated[OpenResponsesStreamEventResponseCompleted, Tag("response.completed")],
        Annotated[
            OpenResponsesStreamEventResponseIncomplete, Tag("response.incomplete")
        ],
        Annotated[OpenResponsesStreamEventResponseFailed, Tag("response.failed")],
        Annotated[OpenResponsesErrorEvent, Tag("error")],
        Annotated[
            OpenResponsesStreamEventResponseOutputItemAdded,
            Tag("response.output_item.added"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseOutputItemDone,
            Tag("response.output_item.done"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseContentPartAdded,
            Tag("response.content_part.added"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseContentPartDone,
            Tag("response.content_part.done"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseOutputTextDelta,
            Tag("response.output_text.delta"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseOutputTextDone,
            Tag("response.output_text.done"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseRefusalDelta, Tag("response.refusal.delta")
        ],
        Annotated[
            OpenResponsesStreamEventResponseRefusalDone, Tag("response.refusal.done")
        ],
        Annotated[
            OpenResponsesStreamEventResponseOutputTextAnnotationAdded,
            Tag("response.output_text.annotation.added"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseFunctionCallArgumentsDelta,
            Tag("response.function_call_arguments.delta"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseFunctionCallArgumentsDone,
            Tag("response.function_call_arguments.done"),
        ],
        Annotated[
            OpenResponsesReasoningDeltaEvent, Tag("response.reasoning_text.delta")
        ],
        Annotated[OpenResponsesReasoningDoneEvent, Tag("response.reasoning_text.done")],
        Annotated[
            OpenResponsesReasoningSummaryPartAddedEvent,
            Tag("response.reasoning_summary_part.added"),
        ],
        Annotated[
            OpenResponsesStreamEventResponseReasoningSummaryPartDone,
            Tag("response.reasoning_summary_part.done"),
        ],
        Annotated[
            OpenResponsesReasoningSummaryTextDeltaEvent,
            Tag("response.reasoning_summary_text.delta"),
        ],
        Annotated[
            OpenResponsesReasoningSummaryTextDoneEvent,
            Tag("response.reasoning_summary_text.done"),
        ],
        Annotated[
            OpenResponsesImageGenCallInProgress,
            Tag("response.image_generation_call.in_progress"),
        ],
        Annotated[
            OpenResponsesImageGenCallGenerating,
            Tag("response.image_generation_call.generating"),
        ],
        Annotated[
            OpenResponsesImageGenCallPartialImage,
            Tag("response.image_generation_call.partial_image"),
        ],
        Annotated[
            OpenResponsesImageGenCallCompleted,
            Tag("response.image_generation_call.completed"),
        ],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""Union of all possible event types emitted during response streaming"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesannotation import (
    OpenAIResponsesAnnotation,
    OpenAIResponsesAnnotationTypedDict,
)
from openrouter.types import BaseModel
import pydantic
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


ResponseOutputTextType = Literal["output_text",]


class ResponseOutputTextTopLogprobTypedDict(TypedDict):
    token: str
    bytes_: List[float]
    logprob: float


class ResponseOutputTextTopLogprob(BaseModel):
    token: str

    bytes_: Annotated[List[float], pydantic.Field(alias="bytes")]

    logprob: float


class LogprobTypedDict(TypedDict):
    token: str
    bytes_: List[float]
    logprob: float
    top_logprobs: List[ResponseOutputTextTopLogprobTypedDict]


class Logprob(BaseModel):
    token: str

    bytes_: Annotated[List[float], pydantic.Field(alias="bytes")]

    logprob: float

    top_logprobs: List[ResponseOutputTextTopLogprob]


class ResponseOutputTextTypedDict(TypedDict):
    type: ResponseOutputTextType
    text: str
    annotations: NotRequired[List[OpenAIResponsesAnnotationTypedDict]]
    logprobs: NotRequired[List[LogprobTypedDict]]


class ResponseOutputText(BaseModel):
    type: ResponseOutputTextType

    text: str

    annotations: Optional[List[OpenAIResponsesAnnotation]] = None

    logprobs: Optional[List[Logprob]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import (
    chatresponse as components_chatresponse,
    chatstreamingresponsechunk as components_chatstreamingresponsechunk,
)
from openrouter.utils import eventstreaming
from typing import Union
from typing_extensions import TypeAliasType


SendChatCompletionRequestResponseTypedDict = TypeAliasType(
    "SendChatCompletionRequestResponseTypedDict",
    Union[
        components_chatresponse.ChatResponseTypedDict,
        Union[
            eventstreaming.EventStream[
                components_chatstreamingresponsechunk.ChatStreamingResponseChunkDataTypedDict
            ],
            eventstreaming.EventStreamAsync[
                components_chatstreamingresponsechunk.ChatStreamingResponseChunkDataTypedDict
            ],
        ],
    ],
)


SendChatCompletionRequestResponse = TypeAliasType(
    "SendChatCompletionRequestResponse",
    Union[
        components_chatresponse.ChatResponse,
        Union[
            eventstreaming.EventStream[
                components_chatstreamingresponsechunk.ChatStreamingResponseChunkData
            ],
            eventstreaming.EventStreamAsync[
                components_chatstreamingresponsechunk.ChatStreamingResponseChunkData
            ],
        ],
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module
import builtins
import sys

if TYPE_CHECKING:
    from .createauthkeyscode import (
        CreateAuthKeysCodeCodeChallengeMethod,
        CreateAuthKeysCodeData,
        CreateAuthKeysCodeDataTypedDict,
        CreateAuthKeysCodeRequest,
        CreateAuthKeysCodeRequestTypedDict,
        CreateAuthKeysCodeResponse,
        CreateAuthKeysCodeResponseTypedDict,
    )
    from .createcoinbasecharge import (
        CallData,
        CallDataTypedDict,
        CreateCoinbaseChargeData,
        CreateCoinbaseChargeDataTypedDict,
        CreateCoinbaseChargeResponse,
        CreateCoinbaseChargeResponseTypedDict,
        CreateCoinbaseChargeSecurity,
        CreateCoinbaseChargeSecurityTypedDict,
        Metadata,
        MetadataTypedDict,
        TransferIntent,
        TransferIntentTypedDict,
        Web3Data,
        Web3DataTypedDict,
    )
    from .createembeddings import (
        Content,
        ContentImageURL,
        ContentImageURLTypedDict,
        ContentText,
        ContentTextTypedDict,
        ContentTypedDict,
        CreateEmbeddingsData,
        CreateEmbeddingsDataTypedDict,
        CreateEmbeddingsRequest,
        CreateEmbeddingsRequestTypedDict,
        CreateEmbeddingsResponse,
        CreateEmbeddingsResponseBody,
        CreateEmbeddingsResponseBodyTypedDict,
        CreateEmbeddingsResponseTypedDict,
        Embedding,
        EmbeddingTypedDict,
        EncodingFormat,
        ImageURL,
        ImageURLTypedDict,
        Input,
        InputTypedDict,
        InputUnion,
        InputUnionTypedDict,
        Object,
        ObjectEmbedding,
        TypeImageURL,
        TypeText,
        Usage,
        UsageTypedDict,
    )
    from .createkeys import (
        CreateKeysData,
        CreateKeysDataTypedDict,
        CreateKeysLimitReset,
        CreateKeysRequest,
        CreateKeysRequestTypedDict,
        CreateKeysResponse,
        CreateKeysResponseTypedDict,
    )
    from .createresponses import (
        CreateResponsesResponse,
        CreateResponsesResponseBody,
        CreateResponsesResponseBodyTypedDict,
        CreateResponsesResponseTypedDict,
    )
    from .deletekeys import (
        DeleteKeysRequest,
        DeleteKeysRequestTypedDict,
        DeleteKeysResponse,
        DeleteKeysResponseTypedDict,
    )
    from .exchangeauthcodeforapikey import (
        ExchangeAuthCodeForAPIKeyCodeChallengeMethod,
        ExchangeAuthCodeForAPIKeyRequest,
        ExchangeAuthCodeForAPIKeyRequestTypedDict,
        ExchangeAuthCodeForAPIKeyResponse,
        ExchangeAuthCodeForAPIKeyResponseTypedDict,
    )
    from .getcredits import (
        GetCreditsData,
        GetCreditsDataTypedDict,
        GetCreditsResponse,
        GetCreditsResponseTypedDict,
    )
    from .getcurrentkey import (
        GetCurrentKeyData,
        GetCurrentKeyDataTypedDict,
        GetCurrentKeyResponse,
        GetCurrentKeyResponseTypedDict,
        RateLimit,
        RateLimitTypedDict,
    )
    from .getgeneration import (
        APIType,
        GetGenerationData,
        GetGenerationDataTypedDict,
        GetGenerationRequest,
        GetGenerationRequestTypedDict,
        GetGenerationResponse,
        GetGenerationResponseTypedDict,
    )
    from .getkey import (
        GetKeyData,
        GetKeyDataTypedDict,
        GetKeyRequest,
        GetKeyRequestTypedDict,
        GetKeyResponse,
        GetKeyResponseTypedDict,
    )
    from .getmodels import GetModelsRequest, GetModelsRequestTypedDict
    from .getparameters import (
        GetParametersData,
        GetParametersDataTypedDict,
        GetParametersRequest,
        GetParametersRequestTypedDict,
        GetParametersResponse,
        GetParametersResponseTypedDict,
        GetParametersSecurity,
        GetParametersSecurityTypedDict,
        SupportedParameter,
    )
    from .getuseractivity import (
        GetUserActivityRequest,
        GetUserActivityRequestTypedDict,
        GetUserActivityResponse,
        GetUserActivityResponseTypedDict,
    )
    from .list import (
        ListData,
        ListDataTypedDict,
        ListRequest,
        ListRequestTypedDict,
        ListResponse,
        ListResponseTypedDict,
    )
    from .listendpoints import (
        ListEndpointsRequest,
        ListEndpointsRequestTypedDict,
        ListEndpointsResponse,
        ListEndpointsResponseTypedDict,
    )
    from .listendpointszdr import (
        ListEndpointsZdrResponse,
        ListEndpointsZdrResponseTypedDict,
    )
    from .listmodelsuser import ListModelsUserSecurity, ListModelsUserSecurityTypedDict
    from .listproviders import (
        ListProvidersData,
        ListProvidersDataTypedDict,
        ListProvidersResponse,
        ListProvidersResponseTypedDict,
    )
    from .sendchatcompletionrequest import (
        SendChatCompletionRequestResponse,
        SendChatCompletionRequestResponseTypedDict,
    )
    from .updatekeys import (
        UpdateKeysData,
        UpdateKeysDataTypedDict,
        UpdateKeysLimitReset,
        UpdateKeysRequest,
        UpdateKeysRequestBody,
        UpdateKeysRequestBodyTypedDict,
        UpdateKeysRequestTypedDict,
        UpdateKeysResponse,
        UpdateKeysResponseTypedDict,
    )

__all__ = [
    "APIType",
    "CallData",
    "CallDataTypedDict",
    "Content",
    "ContentImageURL",
    "ContentImageURLTypedDict",
    "ContentText",
    "ContentTextTypedDict",
    "ContentTypedDict",
    "CreateAuthKeysCodeCodeChallengeMethod",
    "CreateAuthKeysCodeData",
    "CreateAuthKeysCodeDataTypedDict",
    "CreateAuthKeysCodeRequest",
    "CreateAuthKeysCodeRequestTypedDict",
    "CreateAuthKeysCodeResponse",
    "CreateAuthKeysCodeResponseTypedDict",
    "CreateCoinbaseChargeData",
    "CreateCoinbaseChargeDataTypedDict",
    "CreateCoinbaseChargeResponse",
    "CreateCoinbaseChargeResponseTypedDict",
    "CreateCoinbaseChargeSecurity",
    "CreateCoinbaseChargeSecurityTypedDict",
    "CreateEmbeddingsData",
    "CreateEmbeddingsDataTypedDict",
    "CreateEmbeddingsRequest",
    "CreateEmbeddingsRequestTypedDict",
    "CreateEmbeddingsResponse",
    "CreateEmbeddingsResponseBody",
    "CreateEmbeddingsResponseBodyTypedDict",
    "CreateEmbeddingsResponseTypedDict",
    "CreateKeysData",
    "CreateKeysDataTypedDict",
    "CreateKeysLimitReset",
    "CreateKeysRequest",
    "CreateKeysRequestTypedDict",
    "CreateKeysResponse",
    "CreateKeysResponseTypedDict",
    "CreateResponsesResponse",
    "CreateResponsesResponseBody",
    "CreateResponsesResponseBodyTypedDict",
    "CreateResponsesResponseTypedDict",
    "DeleteKeysRequest",
    "DeleteKeysRequestTypedDict",
    "DeleteKeysResponse",
    "DeleteKeysResponseTypedDict",
    "Embedding",
    "EmbeddingTypedDict",
    "EncodingFormat",
    "ExchangeAuthCodeForAPIKeyCodeChallengeMethod",
    "ExchangeAuthCodeForAPIKeyRequest",
    "ExchangeAuthCodeForAPIKeyRequestTypedDict",
    "ExchangeAuthCodeForAPIKeyResponse",
    "ExchangeAuthCodeForAPIKeyResponseTypedDict",
    "GetCreditsData",
    "GetCreditsDataTypedDict",
    "GetCreditsResponse",
    "GetCreditsResponseTypedDict",
    "GetCurrentKeyData",
    "GetCurrentKeyDataTypedDict",
    "GetCurrentKeyResponse",
    "GetCurrentKeyResponseTypedDict",
    "GetGenerationData",
    "GetGenerationDataTypedDict",
    "GetGenerationRequest",
    "GetGenerationRequestTypedDict",
    "GetGenerationResponse",
    "GetGenerationResponseTypedDict",
    "GetKeyData",
    "GetKeyDataTypedDict",
    "GetKeyRequest",
    "GetKeyRequestTypedDict",
    "GetKeyResponse",
    "GetKeyResponseTypedDict",
    "GetModelsRequest",
    "GetModelsRequestTypedDict",
    "GetParametersData",
    "GetParametersDataTypedDict",
    "GetParametersRequest",
    "GetParametersRequestTypedDict",
    "GetParametersResponse",
    "GetParametersResponseTypedDict",
    "GetParametersSecurity",
    "GetParametersSecurityTypedDict",
    "GetUserActivityRequest",
    "GetUserActivityRequestTypedDict",
    "GetUserActivityResponse",
    "GetUserActivityResponseTypedDict",
    "ImageURL",
    "ImageURLTypedDict",
    "Input",
    "InputTypedDict",
    "InputUnion",
    "InputUnionTypedDict",
    "ListData",
    "ListDataTypedDict",
    "ListEndpointsRequest",
    "ListEndpointsRequestTypedDict",
    "ListEndpointsResponse",
    "ListEndpointsResponseTypedDict",
    "ListEndpointsZdrResponse",
    "ListEndpointsZdrResponseTypedDict",
    "ListModelsUserSecurity",
    "ListModelsUserSecurityTypedDict",
    "ListProvidersData",
    "ListProvidersDataTypedDict",
    "ListProvidersResponse",
    "ListProvidersResponseTypedDict",
    "ListRequest",
    "ListRequestTypedDict",
    "ListResponse",
    "ListResponseTypedDict",
    "Metadata",
    "MetadataTypedDict",
    "Object",
    "ObjectEmbedding",
    "RateLimit",
    "RateLimitTypedDict",
    "SendChatCompletionRequestResponse",
    "SendChatCompletionRequestResponseTypedDict",
    "SupportedParameter",
    "TransferIntent",
    "TransferIntentTypedDict",
    "TypeImageURL",
    "TypeText",
    "UpdateKeysData",
    "UpdateKeysDataTypedDict",
    "UpdateKeysLimitReset",
    "UpdateKeysRequest",
    "UpdateKeysRequestBody",
    "UpdateKeysRequestBodyTypedDict",
    "UpdateKeysRequestTypedDict",
    "UpdateKeysResponse",
    "UpdateKeysResponseTypedDict",
    "Usage",
    "UsageTypedDict",
    "Web3Data",
    "Web3DataTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "CreateAuthKeysCodeCodeChallengeMethod": ".createauthkeyscode",
    "CreateAuthKeysCodeData": ".createauthkeyscode",
    "CreateAuthKeysCodeDataTypedDict": ".createauthkeyscode",
    "CreateAuthKeysCodeRequest": ".createauthkeyscode",
    "CreateAuthKeysCodeRequestTypedDict": ".createauthkeyscode",
    "CreateAuthKeysCodeResponse": ".createauthkeyscode",
    "CreateAuthKeysCodeResponseTypedDict": ".createauthkeyscode",
    "CallData": ".createcoinbasecharge",
    "CallDataTypedDict": ".createcoinbasecharge",
    "CreateCoinbaseChargeData": ".createcoinbasecharge",
    "CreateCoinbaseChargeDataTypedDict": ".createcoinbasecharge",
    "CreateCoinbaseChargeResponse": ".createcoinbasecharge",
    "CreateCoinbaseChargeResponseTypedDict": ".createcoinbasecharge",
    "CreateCoinbaseChargeSecurity": ".createcoinbasecharge",
    "CreateCoinbaseChargeSecurityTypedDict": ".createcoinbasecharge",
    "Metadata": ".createcoinbasecharge",
    "MetadataTypedDict": ".createcoinbasecharge",
    "TransferIntent": ".createcoinbasecharge",
    "TransferIntentTypedDict": ".createcoinbasecharge",
    "Web3Data": ".createcoinbasecharge",
    "Web3DataTypedDict": ".createcoinbasecharge",
    "Content": ".createembeddings",
    "ContentImageURL": ".createembeddings",
    "ContentImageURLTypedDict": ".createembeddings",
    "ContentText": ".createembeddings",
    "ContentTextTypedDict": ".createembeddings",
    "ContentTypedDict": ".createembeddings",
    "CreateEmbeddingsData": ".createembeddings",
    "CreateEmbeddingsDataTypedDict": ".createembeddings",
    "CreateEmbeddingsRequest": ".createembeddings",
    "CreateEmbeddingsRequestTypedDict": ".createembeddings",
    "CreateEmbeddingsResponse": ".createembeddings",
    "CreateEmbeddingsResponseBody": ".createembeddings",
    "CreateEmbeddingsResponseBodyTypedDict": ".createembeddings",
    "CreateEmbeddingsResponseTypedDict": ".createembeddings",
    "Embedding": ".createembeddings",
    "EmbeddingTypedDict": ".createembeddings",
    "EncodingFormat": ".createembeddings",
    "ImageURL": ".createembeddings",
    "ImageURLTypedDict": ".createembeddings",
    "Input": ".createembeddings",
    "InputTypedDict": ".createembeddings",
    "InputUnion": ".createembeddings",
    "InputUnionTypedDict": ".createembeddings",
    "Object": ".createembeddings",
    "ObjectEmbedding": ".createembeddings",
    "TypeImageURL": ".createembeddings",
    "TypeText": ".createembeddings",
    "Usage": ".createembeddings",
    "UsageTypedDict": ".createembeddings",
    "CreateKeysData": ".createkeys",
    "CreateKeysDataTypedDict": ".createkeys",
    "CreateKeysLimitReset": ".createkeys",
    "CreateKeysRequest": ".createkeys",
    "CreateKeysRequestTypedDict": ".createkeys",
    "CreateKeysResponse": ".createkeys",
    "CreateKeysResponseTypedDict": ".createkeys",
    "CreateResponsesResponse": ".createresponses",
    "CreateResponsesResponseBody": ".createresponses",
    "CreateResponsesResponseBodyTypedDict": ".createresponses",
    "CreateResponsesResponseTypedDict": ".createresponses",
    "DeleteKeysRequest": ".deletekeys",
    "DeleteKeysRequestTypedDict": ".deletekeys",
    "DeleteKeysResponse": ".deletekeys",
    "DeleteKeysResponseTypedDict": ".deletekeys",
    "ExchangeAuthCodeForAPIKeyCodeChallengeMethod": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyRequest": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyRequestTypedDict": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyResponse": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyResponseTypedDict": ".exchangeauthcodeforapikey",
    "GetCreditsData": ".getcredits",
    "GetCreditsDataTypedDict": ".getcredits",
    "GetCreditsResponse": ".getcredits",
    "GetCreditsResponseTypedDict": ".getcredits",
    "GetCurrentKeyData": ".getcurrentkey",
    "GetCurrentKeyDataTypedDict": ".getcurrentkey",
    "GetCurrentKeyResponse": ".getcurrentkey",
    "GetCurrentKeyResponseTypedDict": ".getcurrentkey",
    "RateLimit": ".getcurrentkey",
    "RateLimitTypedDict": ".getcurrentkey",
    "APIType": ".getgeneration",
    "GetGenerationData": ".getgeneration",
    "GetGenerationDataTypedDict": ".getgeneration",
    "GetGenerationRequest": ".getgeneration",
    "GetGenerationRequestTypedDict": ".getgeneration",
    "GetGenerationResponse": ".getgeneration",
    "GetGenerationResponseTypedDict": ".getgeneration",
    "GetKeyData": ".getkey",
    "GetKeyDataTypedDict": ".getkey",
    "GetKeyRequest": ".getkey",
    "GetKeyRequestTypedDict": ".getkey",
    "GetKeyResponse": ".getkey",
    "GetKeyResponseTypedDict": ".getkey",
    "GetModelsRequest": ".getmodels",
    "GetModelsRequestTypedDict": ".getmodels",
    "GetParametersData": ".getparameters",
    "GetParametersDataTypedDict": ".getparameters",
    "GetParametersRequest": ".getparameters",
    "GetParametersRequestTypedDict": ".getparameters",
    "GetParametersResponse": ".getparameters",
    "GetParametersResponseTypedDict": ".getparameters",
    "GetParametersSecurity": ".getparameters",
    "GetParametersSecurityTypedDict": ".getparameters",
    "SupportedParameter": ".getparameters",
    "GetUserActivityRequest": ".getuseractivity",
    "GetUserActivityRequestTypedDict": ".getuseractivity",
    "GetUserActivityResponse": ".getuseractivity",
    "GetUserActivityResponseTypedDict": ".getuseractivity",
    "ListData": ".list",
    "ListDataTypedDict": ".list",
    "ListRequest": ".list",
    "ListRequestTypedDict": ".list",
    "ListResponse": ".list",
    "ListResponseTypedDict": ".list",
    "ListEndpointsRequest": ".listendpoints",
    "ListEndpointsRequestTypedDict": ".listendpoints",
    "ListEndpointsResponse": ".listendpoints",
    "ListEndpointsResponseTypedDict": ".listendpoints",
    "ListEndpointsZdrResponse": ".listendpointszdr",
    "ListEndpointsZdrResponseTypedDict": ".listendpointszdr",
    "ListModelsUserSecurity": ".listmodelsuser",
    "ListModelsUserSecurityTypedDict": ".listmodelsuser",
    "ListProvidersData": ".listproviders",
    "ListProvidersDataTypedDict": ".listproviders",
    "ListProvidersResponse": ".listproviders",
    "ListProvidersResponseTypedDict": ".listproviders",
    "SendChatCompletionRequestResponse": ".sendchatcompletionrequest",
    "SendChatCompletionRequestResponseTypedDict": ".sendchatcompletionrequest",
    "UpdateKeysData": ".updatekeys",
    "UpdateKeysDataTypedDict": ".updatekeys",
    "UpdateKeysLimitReset": ".updatekeys",
    "UpdateKeysRequest": ".updatekeys",
    "UpdateKeysRequestBody": ".updatekeys",
    "UpdateKeysRequestBodyTypedDict": ".updatekeys",
    "UpdateKeysRequestTypedDict": ".updatekeys",
    "UpdateKeysResponse": ".updatekeys",
    "UpdateKeysResponseTypedDict": ".updatekeys",
}


def dynamic_import(modname, retries=3):
    for attempt in range(retries):
        try:
            return import_module(modname, __package__)
        except KeyError:
            # Clear any half-initialized module and retry
            sys.modules.pop(modname, None)
            if attempt == retries - 1:
                break
    raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = dynamic_import(module_name)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesReasoningDeltaEventType = Literal["response.reasoning_text.delta",]


class OpenResponsesReasoningDeltaEventTypedDict(TypedDict):
    r"""Event emitted when reasoning text delta is streamed"""

    type: OpenResponsesReasoningDeltaEventType
    output_index: float
    item_id: str
    content_index: float
    delta: str
    sequence_number: float


class OpenResponsesReasoningDeltaEvent(BaseModel):
    r"""Event emitted when reasoning text delta is streamed"""

    type: OpenResponsesReasoningDeltaEventType

    output_index: float

    item_id: str

    content_index: float

    delta: str

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from openrouter.utils import FieldMetadata, QueryParamMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetModelsRequestTypedDict(TypedDict):
    category: NotRequired[str]
    supported_parameters: NotRequired[str]


class GetModelsRequest(BaseModel):
    category: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    supported_parameters: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from openrouter.utils import get_discriminator, validate_const
import pydantic
from pydantic import Discriminator, Tag
from pydantic.functional_validators import AfterValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class VideoURL2TypedDict(TypedDict):
    url: str


class VideoURL2(BaseModel):
    url: str


class ChatMessageContentItemVideoVideoURLTypedDict(TypedDict):
    video_url: VideoURL2TypedDict
    type: Literal["video_url"]


class ChatMessageContentItemVideoVideoURL(BaseModel):
    video_url: VideoURL2

    TYPE: Annotated[
        Annotated[Literal["video_url"], AfterValidator(validate_const("video_url"))],
        pydantic.Field(alias="type"),
    ] = "video_url"


class VideoURL1TypedDict(TypedDict):
    url: str


class VideoURL1(BaseModel):
    url: str


class ChatMessageContentItemVideoInputVideoTypedDict(TypedDict):
    video_url: VideoURL1TypedDict
    type: Literal["input_video"]


class ChatMessageContentItemVideoInputVideo(BaseModel):
    video_url: VideoURL1

    TYPE: Annotated[
        Annotated[
            Literal["input_video"], AfterValidator(validate_const("input_video"))
        ],
        pydantic.Field(alias="type"),
    ] = "input_video"


ChatMessageContentItemVideoTypedDict = TypeAliasType(
    "ChatMessageContentItemVideoTypedDict",
    Union[
        ChatMessageContentItemVideoInputVideoTypedDict,
        ChatMessageContentItemVideoVideoURLTypedDict,
    ],
)


ChatMessageContentItemVideo = Annotated[
    Union[
        Annotated[ChatMessageContentItemVideoInputVideo, Tag("input_video")],
        Annotated[ChatMessageContentItemVideoVideoURL, Tag("video_url")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ._schema3 import Schema3, Schema3TypedDict
from .assistantmessage import AssistantMessage, AssistantMessageTypedDict
from .chatcompletionfinishreason import ChatCompletionFinishReason
from .chatmessagetokenlogprobs import (
    ChatMessageTokenLogprobs,
    ChatMessageTokenLogprobsTypedDict,
)
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChatResponseChoiceTypedDict(TypedDict):
    finish_reason: Nullable[ChatCompletionFinishReason]
    index: float
    message: AssistantMessageTypedDict
    reasoning_details: NotRequired[List[Schema3TypedDict]]
    logprobs: NotRequired[Nullable[ChatMessageTokenLogprobsTypedDict]]


class ChatResponseChoice(BaseModel):
    finish_reason: Annotated[
        Nullable[ChatCompletionFinishReason], PlainValidator(validate_open_enum(False))
    ]

    index: float

    message: AssistantMessage

    reasoning_details: Optional[List[Schema3]] = None

    logprobs: OptionalNullable[ChatMessageTokenLogprobs] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["reasoning_details", "logprobs"]
        nullable_fields = ["finish_reason", "logprobs"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_const, validate_open_enum
import pydantic
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


TTL = Union[
    Literal[
        "5m",
        "1h",
    ],
    UnrecognizedStr,
]


class ChatMessageContentItemCacheControlTypedDict(TypedDict):
    type: Literal["ephemeral"]
    ttl: NotRequired[TTL]


class ChatMessageContentItemCacheControl(BaseModel):
    TYPE: Annotated[
        Annotated[Literal["ephemeral"], AfterValidator(validate_const("ephemeral"))],
        pydantic.Field(alias="type"),
    ] = "ephemeral"

    ttl: Annotated[Optional[TTL], PlainValidator(validate_open_enum(False))] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing_extensions import TypedDict


class PerRequestLimitsTypedDict(TypedDict):
    r"""Per-request token limits"""

    prompt_tokens: float
    r"""Maximum prompt tokens per request"""
    completion_tokens: float
    r"""Maximum completion tokens per request"""


class PerRequestLimits(BaseModel):
    r"""Per-request token limits"""

    prompt_tokens: float
    r"""Maximum prompt tokens per request"""

    completion_tokens: float
    r"""Maximum completion tokens per request"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from openrouter import components, errors, operations, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional


class OAuth(BaseSDK):
    r"""OAuth authentication endpoints"""

    def exchange_auth_code_for_api_key(
        self,
        *,
        code: str,
        code_verifier: Optional[str] = None,
        code_challenge_method: OptionalNullable[
            operations.ExchangeAuthCodeForAPIKeyCodeChallengeMethod
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ExchangeAuthCodeForAPIKeyResponse:
        r"""Exchange authorization code for API key

        Exchange an authorization code from the PKCE flow for a user-controlled API key

        :param code: The authorization code received from the OAuth redirect
        :param code_verifier: The code verifier if code_challenge was used in the authorization request
        :param code_challenge_method: The method used to generate the code challenge
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ExchangeAuthCodeForAPIKeyRequest(
            code=code,
            code_verifier=code_verifier,
            code_challenge_method=code_challenge_method,
        )

        req = self._build_request(
            method="POST",
            path="/auth/keys",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                operations.ExchangeAuthCodeForAPIKeyRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="exchangeAuthCodeForAPIKey",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                operations.ExchangeAuthCodeForAPIKeyResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                errors.ForbiddenResponseErrorData, http_res
            )
            raise errors.ForbiddenResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def exchange_auth_code_for_api_key_async(
        self,
        *,
        code: str,
        code_verifier: Optional[str] = None,
        code_challenge_method: OptionalNullable[
            operations.ExchangeAuthCodeForAPIKeyCodeChallengeMethod
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ExchangeAuthCodeForAPIKeyResponse:
        r"""Exchange authorization code for API key

        Exchange an authorization code from the PKCE flow for a user-controlled API key

        :param code: The authorization code received from the OAuth redirect
        :param code_verifier: The code verifier if code_challenge was used in the authorization request
        :param code_challenge_method: The method used to generate the code challenge
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ExchangeAuthCodeForAPIKeyRequest(
            code=code,
            code_verifier=code_verifier,
            code_challenge_method=code_challenge_method,
        )

        req = self._build_request_async(
            method="POST",
            path="/auth/keys",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                operations.ExchangeAuthCodeForAPIKeyRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="exchangeAuthCodeForAPIKey",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                operations.ExchangeAuthCodeForAPIKeyResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                errors.ForbiddenResponseErrorData, http_res
            )
            raise errors.ForbiddenResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    def create_auth_code(
        self,
        *,
        callback_url: str,
        code_challenge: Optional[str] = None,
        code_challenge_method: Optional[
            operations.CreateAuthKeysCodeCodeChallengeMethod
        ] = None,
        limit: Optional[float] = None,
        expires_at: OptionalNullable[datetime] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateAuthKeysCodeResponse:
        r"""Create authorization code

        Create an authorization code for the PKCE flow to generate a user-controlled API key

        :param callback_url: The callback URL to redirect to after authorization. Note, only https URLs on ports 443 and 3000 are allowed.
        :param code_challenge: PKCE code challenge for enhanced security
        :param code_challenge_method: The method used to generate the code challenge
        :param limit: Credit limit for the API key to be created
        :param expires_at: Optional expiration time for the API key to be created
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateAuthKeysCodeRequest(
            callback_url=callback_url,
            code_challenge=code_challenge,
            code_challenge_method=code_challenge_method,
            limit=limit,
            expires_at=expires_at,
        )

        req = self._build_request(
            method="POST",
            path="/auth/keys/code",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateAuthKeysCodeRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createAuthKeysCode",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                operations.CreateAuthKeysCodeResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def create_auth_code_async(
        self,
        *,
        callback_url: str,
        code_challenge: Optional[str] = None,
        code_challenge_method: Optional[
            operations.CreateAuthKeysCodeCodeChallengeMethod
        ] = None,
        limit: Optional[float] = None,
        expires_at: OptionalNullable[datetime] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateAuthKeysCodeResponse:
        r"""Create authorization code

        Create an authorization code for the PKCE flow to generate a user-controlled API key

        :param callback_url: The callback URL to redirect to after authorization. Note, only https URLs on ports 443 and 3000 are allowed.
        :param code_challenge: PKCE code challenge for enhanced security
        :param code_challenge_method: The method used to generate the code challenge
        :param limit: Credit limit for the API key to be created
        :param expires_at: Optional expiration time for the API key to be created
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateAuthKeysCodeRequest(
            callback_url=callback_url,
            code_challenge=code_challenge,
            code_challenge_method=code_challenge_method,
            limit=limit,
            expires_at=expires_at,
        )

        req = self._build_request_async(
            method="POST",
            path="/auth/keys/code",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateAuthKeysCodeRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createAuthKeysCode",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                operations.CreateAuthKeysCodeResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OpenResponsesTopLogprobsTypedDict(TypedDict):
    r"""Alternative token with its log probability"""

    token: NotRequired[str]
    logprob: NotRequired[float]


class OpenResponsesTopLogprobs(BaseModel):
    r"""Alternative token with its log probability"""

    token: Optional[str] = None

    logprob: Optional[float] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .providersort import ProviderSort
from .providersortconfig import ProviderSortConfig, ProviderSortConfigTypedDict
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ProviderSortUnionTypedDict = TypeAliasType(
    "ProviderSortUnionTypedDict", Union[ProviderSortConfigTypedDict, ProviderSort]
)


ProviderSortUnion = TypeAliasType(
    "ProviderSortUnion",
    Union[
        ProviderSortConfig,
        Annotated[ProviderSort, PlainValidator(validate_open_enum(False))],
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesReasoningSummaryTextDeltaEventType = Literal[
    "response.reasoning_summary_text.delta",
]


class OpenResponsesReasoningSummaryTextDeltaEventTypedDict(TypedDict):
    r"""Event emitted when reasoning summary text delta is streamed"""

    type: OpenResponsesReasoningSummaryTextDeltaEventType
    item_id: str
    output_index: float
    summary_index: float
    delta: str
    sequence_number: float


class OpenResponsesReasoningSummaryTextDeltaEvent(BaseModel):
    r"""Event emitted when reasoning summary text delta is streamed"""

    type: OpenResponsesReasoningSummaryTextDeltaEventType

    item_id: str

    output_index: float

    summary_index: float

    delta: str

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .assistantmessage import AssistantMessage, AssistantMessageTypedDict
from .chatmessagecontentitemtext import (
    ChatMessageContentItemText,
    ChatMessageContentItemTextTypedDict,
)
from .systemmessage import SystemMessage, SystemMessageTypedDict
from .toolresponsemessage import ToolResponseMessage, ToolResponseMessageTypedDict
from .usermessage import UserMessage, UserMessageTypedDict
from openrouter.types import BaseModel
from openrouter.utils import get_discriminator, validate_const
import pydantic
from pydantic import Discriminator, Tag
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


MessageContentTypedDict = TypeAliasType(
    "MessageContentTypedDict", Union[str, List[ChatMessageContentItemTextTypedDict]]
)


MessageContent = TypeAliasType(
    "MessageContent", Union[str, List[ChatMessageContentItemText]]
)


class MessageDeveloperTypedDict(TypedDict):
    content: MessageContentTypedDict
    role: Literal["developer"]
    name: NotRequired[str]


class MessageDeveloper(BaseModel):
    content: MessageContent

    ROLE: Annotated[
        Annotated[Literal["developer"], AfterValidator(validate_const("developer"))],
        pydantic.Field(alias="role"),
    ] = "developer"

    name: Optional[str] = None


MessageTypedDict = TypeAliasType(
    "MessageTypedDict",
    Union[
        SystemMessageTypedDict,
        UserMessageTypedDict,
        MessageDeveloperTypedDict,
        ToolResponseMessageTypedDict,
        AssistantMessageTypedDict,
    ],
)


Message = Annotated[
    Union[
        Annotated[SystemMessage, Tag("system")],
        Annotated[UserMessage, Tag("user")],
        Annotated[MessageDeveloper, Tag("developer")],
        Annotated[AssistantMessage, Tag("assistant")],
        Annotated[ToolResponseMessage, Tag("tool")],
    ],
    Discriminator(lambda m: get_discriminator(m, "role", "role")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ._schema3 import Schema3, Schema3TypedDict
from .chatstreamingmessagetoolcall import (
    ChatStreamingMessageToolCall,
    ChatStreamingMessageToolCallTypedDict,
)
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatStreamingMessageChunkRole = Literal["assistant",]


class ChatStreamingMessageChunkTypedDict(TypedDict):
    role: NotRequired[ChatStreamingMessageChunkRole]
    content: NotRequired[Nullable[str]]
    reasoning: NotRequired[Nullable[str]]
    refusal: NotRequired[Nullable[str]]
    tool_calls: NotRequired[List[ChatStreamingMessageToolCallTypedDict]]
    reasoning_details: NotRequired[List[Schema3TypedDict]]


class ChatStreamingMessageChunk(BaseModel):
    role: Optional[ChatStreamingMessageChunkRole] = None

    content: OptionalNullable[str] = UNSET

    reasoning: OptionalNullable[str] = UNSET

    refusal: OptionalNullable[str] = UNSET

    tool_calls: Optional[List[ChatStreamingMessageToolCall]] = None

    reasoning_details: Optional[List[Schema3]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "role",
            "content",
            "reasoning",
            "refusal",
            "tool_calls",
            "reasoning_details",
        ]
        nullable_fields = ["content", "reasoning", "refusal"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_const, validate_open_enum
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


Schema5 = Union[
    Literal[
        "unknown",
        "openai-responses-v1",
        "azure-openai-responses-v1",
        "xai-responses-v1",
        "anthropic-claude-v1",
        "google-gemini-v1",
    ],
    UnrecognizedStr,
]


class Schema3ReasoningTextTypedDict(TypedDict):
    type: Literal["reasoning.text"]
    text: NotRequired[Nullable[str]]
    signature: NotRequired[Nullable[str]]
    id: NotRequired[Nullable[str]]
    format_: NotRequired[Nullable[Schema5]]
    index: NotRequired[float]


class Schema3ReasoningText(BaseModel):
    TYPE: Annotated[
        Annotated[
            Literal["reasoning.text"], AfterValidator(validate_const("reasoning.text"))
        ],
        pydantic.Field(alias="type"),
    ] = "reasoning.text"

    text: OptionalNullable[str] = UNSET

    signature: OptionalNullable[str] = UNSET

    id: OptionalNullable[str] = UNSET

    format_: Annotated[
        Annotated[OptionalNullable[Schema5], PlainValidator(validate_open_enum(False))],
        pydantic.Field(alias="format"),
    ] = UNSET

    index: Optional[float] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["text", "signature", "id", "format", "index"]
        nullable_fields = ["text", "signature", "id", "format"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class Schema3ReasoningEncryptedTypedDict(TypedDict):
    data: str
    type: Literal["reasoning.encrypted"]
    id: NotRequired[Nullable[str]]
    format_: NotRequired[Nullable[Schema5]]
    index: NotRequired[float]


class Schema3ReasoningEncrypted(BaseModel):
    data: str

    TYPE: Annotated[
        Annotated[
            Literal["reasoning.encrypted"],
            AfterValidator(validate_const("reasoning.encrypted")),
        ],
        pydantic.Field(alias="type"),
    ] = "reasoning.encrypted"

    id: OptionalNullable[str] = UNSET

    format_: Annotated[
        Annotated[OptionalNullable[Schema5], PlainValidator(validate_open_enum(False))],
        pydantic.Field(alias="format"),
    ] = UNSET

    index: Optional[float] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "format", "index"]
        nullable_fields = ["id", "format"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class Schema3ReasoningSummaryTypedDict(TypedDict):
    summary: str
    type: Literal["reasoning.summary"]
    id: NotRequired[Nullable[str]]
    format_: NotRequired[Nullable[Schema5]]
    index: NotRequired[float]


class Schema3ReasoningSummary(BaseModel):
    summary: str

    TYPE: Annotated[
        Annotated[
            Literal["reasoning.summary"],
            AfterValidator(validate_const("reasoning.summary")),
        ],
        pydantic.Field(alias="type"),
    ] = "reasoning.summary"

    id: OptionalNullable[str] = UNSET

    format_: Annotated[
        Annotated[OptionalNullable[Schema5], PlainValidator(validate_open_enum(False))],
        pydantic.Field(alias="format"),
    ] = UNSET

    index: Optional[float] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "format", "index"]
        nullable_fields = ["id", "format"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


Schema3TypedDict = TypeAliasType(
    "Schema3TypedDict",
    Union[
        Schema3ReasoningSummaryTypedDict,
        Schema3ReasoningEncryptedTypedDict,
        Schema3ReasoningTextTypedDict,
    ],
)


Schema3 = Annotated[
    Union[
        Annotated[Schema3ReasoningSummary, Tag("reasoning.summary")],
        Annotated[Schema3ReasoningEncrypted, Tag("reasoning.encrypted")],
        Annotated[Schema3ReasoningText, Tag("reasoning.text")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from typing import Literal, Union


ProviderName = Union[
    Literal[
        "AI21",
        "AionLabs",
        "Alibaba",
        "Amazon Bedrock",
        "Amazon Nova",
        "Anthropic",
        "Arcee AI",
        "AtlasCloud",
        "Avian",
        "Azure",
        "BaseTen",
        "BytePlus",
        "Black Forest Labs",
        "Cerebras",
        "Chutes",
        "Cirrascale",
        "Clarifai",
        "Cloudflare",
        "Cohere",
        "Crusoe",
        "DeepInfra",
        "DeepSeek",
        "Featherless",
        "Fireworks",
        "Friendli",
        "GMICloud",
        "Google",
        "Google AI Studio",
        "Groq",
        "Hyperbolic",
        "Inception",
        "Inceptron",
        "InferenceNet",
        "Infermatic",
        "Inflection",
        "Liquid",
        "Mara",
        "Mancer 2",
        "Minimax",
        "ModelRun",
        "Mistral",
        "Modular",
        "Moonshot AI",
        "Morph",
        "NCompass",
        "Nebius",
        "NextBit",
        "Novita",
        "Nvidia",
        "OpenAI",
        "OpenInference",
        "Parasail",
        "Perplexity",
        "Phala",
        "Relace",
        "SambaNova",
        "Seed",
        "SiliconFlow",
        "Sourceful",
        "Stealth",
        "StreamLake",
        "Switchpoint",
        "Together",
        "Upstage",
        "Venice",
        "WandB",
        "Xiaomi",
        "xAI",
        "Z.AI",
        "FakeProvider",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing_extensions import TypedDict


class PercentileStatsTypedDict(TypedDict):
    r"""Latency percentiles in milliseconds over the last 30 minutes. Latency measures time to first token. Only visible when authenticated with an API key or cookie; returns null for unauthenticated requests."""

    p50: float
    r"""Median (50th percentile)"""
    p75: float
    r"""75th percentile"""
    p90: float
    r"""90th percentile"""
    p99: float
    r"""99th percentile"""


class PercentileStats(BaseModel):
    r"""Latency percentiles in milliseconds over the last 30 minutes. Latency measures time to first token. Only visible when authenticated with an API key or cookie; returns null for unauthenticated requests."""

    p50: float
    r"""Median (50th percentile)"""

    p75: float
    r"""75th percentile"""

    p90: float
    r"""90th percentile"""

    p99: float
    r"""99th percentile"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from ._schema0 import Schema0, Schema0TypedDict
from .chatstreamoptions import ChatStreamOptions, ChatStreamOptionsTypedDict
from .message import Message, MessageTypedDict
from .providersortunion import ProviderSortUnion, ProviderSortUnionTypedDict
from .reasoningsummaryverbosity import ReasoningSummaryVerbosity
from .responseformatjsonschema import (
    ResponseFormatJSONSchema,
    ResponseFormatJSONSchemaTypedDict,
)
from .responseformattextgrammar import (
    ResponseFormatTextGrammar,
    ResponseFormatTextGrammarTypedDict,
)
from .tooldefinitionjson import ToolDefinitionJSON, ToolDefinitionJSONTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_const, validate_open_enum
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ChatGenerationParamsDataCollection = Union[
    Literal[
        "deny",
        "allow",
    ],
    UnrecognizedStr,
]


Quantizations = Union[
    Literal[
        "int4",
        "int8",
        "fp4",
        "fp6",
        "fp8",
        "fp16",
        "bf16",
        "fp32",
        "unknown",
    ],
    UnrecognizedStr,
]


class ChatGenerationParamsMaxPriceTypedDict(TypedDict):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: NotRequired[Any]
    completion: NotRequired[Any]
    image: NotRequired[Any]
    audio: NotRequired[Any]
    request: NotRequired[Any]


class ChatGenerationParamsMaxPrice(BaseModel):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: Optional[Any] = None

    completion: Optional[Any] = None

    image: Optional[Any] = None

    audio: Optional[Any] = None

    request: Optional[Any] = None


class ChatGenerationParamsPreferredMinThroughputTypedDict(TypedDict):
    p50: NotRequired[Nullable[float]]
    p75: NotRequired[Nullable[float]]
    p90: NotRequired[Nullable[float]]
    p99: NotRequired[Nullable[float]]


class ChatGenerationParamsPreferredMinThroughput(BaseModel):
    p50: OptionalNullable[float] = UNSET

    p75: OptionalNullable[float] = UNSET

    p90: OptionalNullable[float] = UNSET

    p99: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["p50", "p75", "p90", "p99"]
        nullable_fields = ["p50", "p75", "p90", "p99"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


ChatGenerationParamsPreferredMinThroughputUnionTypedDict = TypeAliasType(
    "ChatGenerationParamsPreferredMinThroughputUnionTypedDict",
    Union[ChatGenerationParamsPreferredMinThroughputTypedDict, float],
)


ChatGenerationParamsPreferredMinThroughputUnion = TypeAliasType(
    "ChatGenerationParamsPreferredMinThroughputUnion",
    Union[ChatGenerationParamsPreferredMinThroughput, float],
)


class ChatGenerationParamsPreferredMaxLatencyTypedDict(TypedDict):
    p50: NotRequired[Nullable[float]]
    p75: NotRequired[Nullable[float]]
    p90: NotRequired[Nullable[float]]
    p99: NotRequired[Nullable[float]]


class ChatGenerationParamsPreferredMaxLatency(BaseModel):
    p50: OptionalNullable[float] = UNSET

    p75: OptionalNullable[float] = UNSET

    p90: OptionalNullable[float] = UNSET

    p99: OptionalNullable[float] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["p50", "p75", "p90", "p99"]
        nullable_fields = ["p50", "p75", "p90", "p99"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


ChatGenerationParamsPreferredMaxLatencyUnionTypedDict = TypeAliasType(
    "ChatGenerationParamsPreferredMaxLatencyUnionTypedDict",
    Union[ChatGenerationParamsPreferredMaxLatencyTypedDict, float],
)


ChatGenerationParamsPreferredMaxLatencyUnion = TypeAliasType(
    "ChatGenerationParamsPreferredMaxLatencyUnion",
    Union[ChatGenerationParamsPreferredMaxLatency, float],
)


class ChatGenerationParamsProviderTypedDict(TypedDict):
    allow_fallbacks: NotRequired[Nullable[bool]]
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """
    require_parameters: NotRequired[Nullable[bool]]
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""
    data_collection: NotRequired[Nullable[ChatGenerationParamsDataCollection]]
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """
    zdr: NotRequired[Nullable[bool]]
    enforce_distillable_text: NotRequired[Nullable[bool]]
    order: NotRequired[Nullable[List[Schema0TypedDict]]]
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""
    only: NotRequired[Nullable[List[Schema0TypedDict]]]
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""
    ignore: NotRequired[Nullable[List[Schema0TypedDict]]]
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""
    quantizations: NotRequired[Nullable[List[Quantizations]]]
    r"""A list of quantization levels to filter the provider by."""
    sort: NotRequired[Nullable[ProviderSortUnionTypedDict]]
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""
    max_price: NotRequired[ChatGenerationParamsMaxPriceTypedDict]
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""
    preferred_min_throughput: NotRequired[
        Nullable[ChatGenerationParamsPreferredMinThroughputUnionTypedDict]
    ]
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""
    preferred_max_latency: NotRequired[
        Nullable[ChatGenerationParamsPreferredMaxLatencyUnionTypedDict]
    ]
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""


class ChatGenerationParamsProvider(BaseModel):
    allow_fallbacks: OptionalNullable[bool] = UNSET
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """

    require_parameters: OptionalNullable[bool] = UNSET
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""

    data_collection: Annotated[
        OptionalNullable[ChatGenerationParamsDataCollection],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """

    zdr: OptionalNullable[bool] = UNSET

    enforce_distillable_text: OptionalNullable[bool] = UNSET

    order: OptionalNullable[List[Schema0]] = UNSET
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""

    only: OptionalNullable[List[Schema0]] = UNSET
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""

    ignore: OptionalNullable[List[Schema0]] = UNSET
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""

    quantizations: OptionalNullable[
        List[Annotated[Quantizations, PlainValidator(validate_open_enum(False))]]
    ] = UNSET
    r"""A list of quantization levels to filter the provider by."""

    sort: OptionalNullable[ProviderSortUnion] = UNSET
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""

    max_price: Optional[ChatGenerationParamsMaxPrice] = None
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    preferred_min_throughput: OptionalNullable[
        ChatGenerationParamsPreferredMinThroughputUnion
    ] = UNSET
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    preferred_max_latency: OptionalNullable[
        ChatGenerationParamsPreferredMaxLatencyUnion
    ] = UNSET
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "max_price",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        nullable_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ChatGenerationParamsPluginResponseHealingTypedDict(TypedDict):
    id: Literal["response-healing"]
    enabled: NotRequired[bool]


class ChatGenerationParamsPluginResponseHealing(BaseModel):
    ID: Annotated[
        Annotated[
            Literal["response-healing"],
            AfterValidator(validate_const("response-healing")),
        ],
        pydantic.Field(alias="id"),
    ] = "response-healing"

    enabled: Optional[bool] = None


PdfEngine = Union[
    Literal[
        "mistral-ocr",
        "pdf-text",
        "native",
    ],
    UnrecognizedStr,
]


class PdfTypedDict(TypedDict):
    engine: NotRequired[PdfEngine]


class Pdf(BaseModel):
    engine: Annotated[
        Optional[PdfEngine], PlainValidator(validate_open_enum(False))
    ] = None


class ChatGenerationParamsPluginFileParserTypedDict(TypedDict):
    id: Literal["file-parser"]
    enabled: NotRequired[bool]
    pdf: NotRequired[PdfTypedDict]


class ChatGenerationParamsPluginFileParser(BaseModel):
    ID: Annotated[
        Annotated[
            Literal["file-parser"], AfterValidator(validate_const("file-parser"))
        ],
        pydantic.Field(alias="id"),
    ] = "file-parser"

    enabled: Optional[bool] = None

    pdf: Optional[Pdf] = None


Engine = Union[
    Literal[
        "native",
        "exa",
    ],
    UnrecognizedStr,
]


class ChatGenerationParamsPluginWebTypedDict(TypedDict):
    id: Literal["web"]
    enabled: NotRequired[bool]
    max_results: NotRequired[float]
    search_prompt: NotRequired[str]
    engine: NotRequired[Engine]


class ChatGenerationParamsPluginWeb(BaseModel):
    ID: Annotated[
        Annotated[Literal["web"], AfterValidator(validate_const("web"))],
        pydantic.Field(alias="id"),
    ] = "web"

    enabled: Optional[bool] = None

    max_results: Optional[float] = None

    search_prompt: Optional[str] = None

    engine: Annotated[Optional[Engine], PlainValidator(validate_open_enum(False))] = (
        None
    )


class ChatGenerationParamsPluginModerationTypedDict(TypedDict):
    id: Literal["moderation"]


class ChatGenerationParamsPluginModeration(BaseModel):
    ID: Annotated[
        Annotated[Literal["moderation"], AfterValidator(validate_const("moderation"))],
        pydantic.Field(alias="id"),
    ] = "moderation"


class ChatGenerationParamsPluginAutoRouterTypedDict(TypedDict):
    id: Literal["auto-router"]
    enabled: NotRequired[bool]
    allowed_models: NotRequired[List[str]]


class ChatGenerationParamsPluginAutoRouter(BaseModel):
    ID: Annotated[
        Annotated[
            Literal["auto-router"], AfterValidator(validate_const("auto-router"))
        ],
        pydantic.Field(alias="id"),
    ] = "auto-router"

    enabled: Optional[bool] = None

    allowed_models: Optional[List[str]] = None


ChatGenerationParamsPluginUnionTypedDict = TypeAliasType(
    "ChatGenerationParamsPluginUnionTypedDict",
    Union[
        ChatGenerationParamsPluginModerationTypedDict,
        ChatGenerationParamsPluginResponseHealingTypedDict,
        ChatGenerationParamsPluginAutoRouterTypedDict,
        ChatGenerationParamsPluginFileParserTypedDict,
        ChatGenerationParamsPluginWebTypedDict,
    ],
)


ChatGenerationParamsPluginUnion = Annotated[
    Union[
        Annotated[ChatGenerationParamsPluginAutoRouter, Tag("auto-router")],
        Annotated[ChatGenerationParamsPluginModeration, Tag("moderation")],
        Annotated[ChatGenerationParamsPluginWeb, Tag("web")],
        Annotated[ChatGenerationParamsPluginFileParser, Tag("file-parser")],
        Annotated[ChatGenerationParamsPluginResponseHealing, Tag("response-healing")],
    ],
    Discriminator(lambda m: get_discriminator(m, "id", "id")),
]


Route = Union[
    Literal[
        "fallback",
        "sort",
    ],
    UnrecognizedStr,
]


Effort = Union[
    Literal[
        "xhigh",
        "high",
        "medium",
        "low",
        "minimal",
        "none",
    ],
    UnrecognizedStr,
]


class ReasoningTypedDict(TypedDict):
    effort: NotRequired[Nullable[Effort]]
    summary: NotRequired[Nullable[ReasoningSummaryVerbosity]]


class Reasoning(BaseModel):
    effort: Annotated[
        OptionalNullable[Effort], PlainValidator(validate_open_enum(False))
    ] = UNSET

    summary: Annotated[
        OptionalNullable[ReasoningSummaryVerbosity],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["effort", "summary"]
        nullable_fields = ["effort", "summary"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ChatGenerationParamsResponseFormatPythonTypedDict(TypedDict):
    type: Literal["python"]


class ChatGenerationParamsResponseFormatPython(BaseModel):
    TYPE: Annotated[
        Annotated[Literal["python"], AfterValidator(validate_const("python"))],
        pydantic.Field(alias="type"),
    ] = "python"


class ChatGenerationParamsResponseFormatJSONObjectTypedDict(TypedDict):
    type: Literal["json_object"]


class ChatGenerationParamsResponseFormatJSONObject(BaseModel):
    TYPE: Annotated[
        Annotated[
            Literal["json_object"], AfterValidator(validate_const("json_object"))
        ],
        pydantic.Field(alias="type"),
    ] = "json_object"


class ChatGenerationParamsResponseFormatTextTypedDict(TypedDict):
    type: Literal["text"]


class ChatGenerationParamsResponseFormatText(BaseModel):
    TYPE: Annotated[
        Annotated[Literal["text"], AfterValidator(validate_const("text"))],
        pydantic.Field(alias="type"),
    ] = "text"


ChatGenerationParamsResponseFormatUnionTypedDict = TypeAliasType(
    "ChatGenerationParamsResponseFormatUnionTypedDict",
    Union[
        ChatGenerationParamsResponseFormatTextTypedDict,
        ChatGenerationParamsResponseFormatJSONObjectTypedDict,
        ChatGenerationParamsResponseFormatPythonTypedDict,
        ResponseFormatJSONSchemaTypedDict,
        ResponseFormatTextGrammarTypedDict,
    ],
)


ChatGenerationParamsResponseFormatUnion = Annotated[
    Union[
        Annotated[ChatGenerationParamsResponseFormatText, Tag("text")],
        Annotated[ChatGenerationParamsResponseFormatJSONObject, Tag("json_object")],
        Annotated[ResponseFormatJSONSchema, Tag("json_schema")],
        Annotated[ResponseFormatTextGrammar, Tag("grammar")],
        Annotated[ChatGenerationParamsResponseFormatPython, Tag("python")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


ChatGenerationParamsStopTypedDict = TypeAliasType(
    "ChatGenerationParamsStopTypedDict", Union[str, List[str]]
)


ChatGenerationParamsStop = TypeAliasType(
    "ChatGenerationParamsStop", Union[str, List[str]]
)


class DebugTypedDict(TypedDict):
    echo_upstream_body: NotRequired[bool]


class Debug(BaseModel):
    echo_upstream_body: Optional[bool] = None


ChatGenerationParamsImageConfigTypedDict = TypeAliasType(
    "ChatGenerationParamsImageConfigTypedDict", Union[str, float]
)


ChatGenerationParamsImageConfig = TypeAliasType(
    "ChatGenerationParamsImageConfig", Union[str, float]
)


Modality = Union[
    Literal[
        "text",
        "image",
    ],
    UnrecognizedStr,
]


class ChatGenerationParamsTypedDict(TypedDict):
    messages: List[MessageTypedDict]
    provider: NotRequired[Nullable[ChatGenerationParamsProviderTypedDict]]
    r"""When multiple model providers are available, optionally indicate your routing preference."""
    plugins: NotRequired[List[ChatGenerationParamsPluginUnionTypedDict]]
    r"""Plugins you want to enable for this request, including their settings."""
    route: NotRequired[Nullable[Route]]
    user: NotRequired[str]
    session_id: NotRequired[str]
    r"""A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters."""
    model: NotRequired[str]
    models: NotRequired[List[str]]
    frequency_penalty: NotRequired[Nullable[float]]
    logit_bias: NotRequired[Nullable[Dict[str, float]]]
    logprobs: NotRequired[Nullable[bool]]
    top_logprobs: NotRequired[Nullable[float]]
    max_completion_tokens: NotRequired[Nullable[float]]
    max_tokens: NotRequired[Nullable[float]]
    metadata: NotRequired[Dict[str, str]]
    presence_penalty: NotRequired[Nullable[float]]
    reasoning: NotRequired[ReasoningTypedDict]
    response_format: NotRequired[ChatGenerationParamsResponseFormatUnionTypedDict]
    seed: NotRequired[Nullable[int]]
    stop: NotRequired[Nullable[ChatGenerationParamsStopTypedDict]]
    stream: NotRequired[bool]
    stream_options: NotRequired[Nullable[ChatStreamOptionsTypedDict]]
    temperature: NotRequired[Nullable[float]]
    tool_choice: NotRequired[Any]
    tools: NotRequired[List[ToolDefinitionJSONTypedDict]]
    top_p: NotRequired[Nullable[float]]
    debug: NotRequired[DebugTypedDict]
    image_config: NotRequired[Dict[str, ChatGenerationParamsImageConfigTypedDict]]
    modalities: NotRequired[List[Modality]]


class ChatGenerationParams(BaseModel):
    messages: List[Message]

    provider: OptionalNullable[ChatGenerationParamsProvider] = UNSET
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    plugins: Optional[List[ChatGenerationParamsPluginUnion]] = None
    r"""Plugins you want to enable for this request, including their settings."""

    route: Annotated[
        OptionalNullable[Route], PlainValidator(validate_open_enum(False))
    ] = UNSET

    user: Optional[str] = None

    session_id: Optional[str] = None
    r"""A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 128 characters."""

    model: Optional[str] = None

    models: Optional[List[str]] = None

    frequency_penalty: OptionalNullable[float] = UNSET

    logit_bias: OptionalNullable[Dict[str, float]] = UNSET

    logprobs: OptionalNullable[bool] = UNSET

    top_logprobs: OptionalNullable[float] = UNSET

    max_completion_tokens: OptionalNullable[float] = UNSET

    max_tokens: OptionalNullable[float] = UNSET

    metadata: Optional[Dict[str, str]] = None

    presence_penalty: OptionalNullable[float] = UNSET

    reasoning: Optional[Reasoning] = None

    response_format: Optional[ChatGenerationParamsResponseFormatUnion] = None

    seed: OptionalNullable[int] = UNSET

    stop: OptionalNullable[ChatGenerationParamsStop] = UNSET

    stream: Optional[bool] = False

    stream_options: OptionalNullable[ChatStreamOptions] = UNSET

    temperature: OptionalNullable[float] = UNSET

    tool_choice: Optional[Any] = None

    tools: Optional[List[ToolDefinitionJSON]] = None

    top_p: OptionalNullable[float] = UNSET

    debug: Optional[Debug] = None

    image_config: Optional[Dict[str, ChatGenerationParamsImageConfig]] = None

    modalities: Optional[
        List[Annotated[Modality, PlainValidator(validate_open_enum(False))]]
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "provider",
            "plugins",
            "route",
            "user",
            "session_id",
            "model",
            "models",
            "frequency_penalty",
            "logit_bias",
            "logprobs",
            "top_logprobs",
            "max_completion_tokens",
            "max_tokens",
            "metadata",
            "presence_penalty",
            "reasoning",
            "response_format",
            "seed",
            "stop",
            "stream",
            "stream_options",
            "temperature",
            "tool_choice",
            "tools",
            "top_p",
            "debug",
            "image_config",
            "modalities",
        ]
        nullable_fields = [
            "provider",
            "route",
            "frequency_penalty",
            "logit_bias",
            "logprobs",
            "top_logprobs",
            "max_completion_tokens",
            "max_tokens",
            "presence_penalty",
            "seed",
            "stop",
            "stream_options",
            "temperature",
            "top_p",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
